#!/bin/sh
EXE=advertise-arm
KERNEL=uImage-advertise*
LOG=/data/log/advertise-arm.log
M3HEX=AD1.hex
STM_FIRMWARE_M3=Ad1_M3_N32.hex
PORTNAME="/dev/ttyS8"

/usr/bin/killall rcS
/usr/bin/killall advertise-arm

rm -f /data/log/finder*
rm -f /data/log/inventor*
rm -f /data/log/guider*

rm -rf /opt/backup
rm -f /opt/md5*

rm -rf /data/pics
mkdir /data/pics

WORKDIR=$(cd `dirname $0`; pwd)
echo "WORKDIR="$WORKDIR >> $LOG

MACHINE_ARCH=armv5tejl

#check ARCH
CHECK_ARCH=`uname -m`
if [ "${MACHINE_ARCH}" != "${CHECK_ARCH}" ];then
	echo "Machine architecture error. ARCH=${CHECK_ARCH}" >> $LOG
	sync
	cat $WORKDIR/failure.bmp > /dev/fb0
	exit 1
fi

echo "Machine architecture ARCH=${CHECK_ARCH}" >> $LOG

VERSION_CONTROL=""
VERSION_KERNEL=""
VERSION_LIBRARY=""
VERSION_SOFTWARE=""

CONTROL=`cat $WORKDIR/version.cfg | grep "control" | tr -s ' ' | cut -d '=' -f2`
if [ ! $CONTROL == "" ] && [ ${#CONTROL} != 0 ]; then
	VERSION_CONTROL=$CONTROL
fi
echo "VERSION_CONTROL="$VERSION_CONTROL >> $LOG

KERNEL_V=`cat $WORKDIR/version.cfg | grep "kernel" | tr -s ' ' | cut -d '=' -f2`
if [ ! $KERNEL_V == "" ] && [ ${#KERNEL_V} != 0 ]; then
	VERSION_KERNEL=$KERNEL_V
fi
echo "VERSION_KERNEL="$VERSION_KERNEL >> $LOG

LIBRARY=`cat $WORKDIR/version.cfg | grep "library" | tr -s ' ' | cut -d '=' -f2`
if [ ! $LIBRARY == "" ] && [ ${#LIBRARY} != 0 ]; then
	VERSION_LIBRARY=$LIBRARY
fi
echo "VERSION_LIBRARY="$VERSION_LIBRARY >> $LOG

SOFTWARE=`cat $WORKDIR/version.cfg | grep "software" | tr -s ' ' | cut -d '=' -f2`
if [ ! $SOFTWARE == "" ] && [ ${#SOFTWARE} != 0 ]; then
	VERSION_SOFTWARE=$SOFTWARE
fi
echo "VERSION_SOFTWARE="$VERSION_SOFTWARE >> $LOG

if [ -f $WORKDIR/pointercal ]; then
    echo "pointercal to board" >> $LOG
    cp -vf $WORKDIR/pointercal /opt/tslib-1.4-none/etc/
    sync
fi

if [ -f $WORKDIR/authorized_keys ]; then
    echo "authorized_keys to board" >> $LOG
    mkdir /root/.ssh
    cp -vf $WORKDIR/authorized_keys /root/.ssh/
    sync
fi

if [ -f $WORKDIR/sshd_config ]; then
    echo "sshd_config to board" >> $LOG
    cp -vf $WORKDIR/sshd_config /etc/ssh/
    sync
fi

if [ -f $WORKDIR/shadow ]; then
    echo "shadow to board" >> $LOG
    cp -vf $WORKDIR/shadow /etc/
    sync
fi

echo "dispaly start on lcd" >> $LOG
if [ -f $WORKDIR/start.bmp ]; then
    cat $WORKDIR/start.bmp > /dev/fb0
fi

if [ ! -d /data/log ]; then
	mkdir /data/log
fi

if [ ! -d /data/pics ]; then
	mkdir /data/pics
fi

if [ ! -d /data/update ]; then
	mkdir /data/update
fi

if [ ! -d /media ]; then
	mkdir /media
fi

mkdir /opt/flashforge


if [ ! -d /PROGRAM ]; then
	mkdir /PROGRAM /PROGRAM/control /PROGRAM/kernel /PROGRAM/library /PROGRAM/software
fi

echo "FlashForge Cloud Switch OFF" >> $LOG
sed -i '1c OFF' /opt/flashforge/cloud-ff
sync


if [ -f $WORKDIR/$KERNEL ] && [ ! $VERSION_KERNEL == "" ]; then
	UIMAGE=`ls -1t $WORKDIR/$KERNEL | head -n 1`
	if [ -f $UIMAGE ]; then
		VER_KERNEL=`ls /PROGRAM/kernel | head -n 1`
		if [ ! $VER_KERNEL ] || [ ! $VER_KERNEL == $VERSION_KERNEL ] || [ $WORKDIR == "/mnt" ]; then
			echo "rm -rf /PROGRAM/kernel/*" >> $LOG
			rm -rf /PROGRAM/kernel/*
			
			echo "create /PROGRAM/kernel/"$VERSION_KERNEL >> $LOG
			mkdir -p /PROGRAM/kernel/$VERSION_KERNEL
			cp -vf $UIMAGE /PROGRAM/kernel/$VERSION_KERNEL
			sync
			
			if [ ! -d /tmp/disk1 ]; then
				mkdir /tmp/disk1
			fi
			mount -t vfat -o codepage=936,iocharset=utf8 /dev/mmcblk0p1 /tmp/disk1
			if [ $? -ne 0 ]; then
				echo "mount mmcblk0p1 failed" >> $LOG
				sync
				exit -1
				halt -f
			fi
			
			echo "update uImage "$UIMAGE >> $LOG
			rm -f /tmp/disk1/uImage
			cp -vf /PROGRAM/kernel/$VERSION_KERNEL/* /tmp/disk1/uImage
			sync
			echo "update uImage success" >> $LOG

			umount /tmp/disk1
			rm -r /tmp/disk1
			
			cd /PROGRAM/kernel
			ls | grep -v $VERSION_KERNEL | xargs rm -rf
		fi
	fi
fi

sync

if [ -f $WORKDIR/auto_run.sh ]; then
	AUTORUN=$WORKDIR/auto_run.sh
	echo "update auto_run.sh "$AUTORUN >> $LOG
	rm -f /opt/auto_run.sh
	cp -vf $AUTORUN /opt
	sync
	chmod a+x /opt/auto_run.sh
	echo "update auto_run.sh success" >> $LOG
fi

sync

if [ -f $WORKDIR/$EXE ] && [ ! $VERSION_SOFTWARE == "" ]; then
	ARM=$WORKDIR/$EXE
	
	VER_SOFTWARE=`ls /PROGRAM/software | head -n 1`
	if [ ! $VER_SOFTWARE ] || [ ! $VER_SOFTWARE == $VERSION_SOFTWARE ] || [ $WORKDIR == "/mnt" ]; then
		echo "rm -rf /PROGRAM/software/*" >> $LOG
		rm -rf /PROGRAM/software/*
		
		echo "create /PROGRAM/software/temp" >> $LOG
		mkdir -p /PROGRAM/software/temp

		echo "update EXE "$EXE >> $LOG
		rm -f /data/log/core*
		rm -f /data/log/strace*

		cp -vf $ARM /PROGRAM/software/temp
		sync
		chmod a+x /PROGRAM/software/temp/$EXE

		mv /PROGRAM/software/temp /PROGRAM/software/$VERSION_SOFTWARE
		
		cd /data/log
		rm -f `ls | grep -v $EXE`

		echo "update EXE success" >> $LOG
	fi
fi

sync

if [ -f $WORKDIR/update/env.cfg ] && [ ! $VERSION_LIBRARY == "" ]; then
	VER_LIBRARY=`ls /PROGRAM/library | head -n 1`
	if [ ! $VER_LIBRARY ] || [ ! $VER_LIBRARY == $VERSION_LIBRARY ] || [ $WORKDIR == "/mnt" ]; then
		echo "rm -rf /PROGRAM/library/*" >> $LOG
		rm -rf /PROGRAM/library/*
		
		echo "create /PROGRAM/library/temp" >> $LOG
		mkdir -p /PROGRAM/library/temp
		
		cp -vf $WORKDIR/update/env.cfg /PROGRAM/library/temp
		sync

		if [ -f $WORKDIR/update/fonts/code2000.ttf ]; then
			echo "update fonts" >> $LOG
			cp -vPrf $WORKDIR/update/fonts /PROGRAM/library/temp
			sync
			echo "update code2000.ttf success" >> $LOG
		fi

		sync

		if [ -f $WORKDIR/update/arm-jsoncpp.tar ]; then
			ARM_JSONCPP=$WORKDIR/update/arm-jsoncpp.tar
			echo "update arm-jsoncpp" >> $LOG
			echo "tar arm-jsoncpp.tar" >> $LOG 
			rm -rf /PROGRAM/library/temp/arm-jsoncpp
			tar -xvf $WORKDIR/update/arm-jsoncpp.tar -C /PROGRAM/library/temp
			echo "tar arm-jsoncpp.tar ret=$?" >> $LOG
			if [ $? == 0 ] || [ $? == 141 ]; then
				echo "update arm-jsoncpp success" >> $LOG
			else
				echo "update arm-jsoncpp failure" >> $LOG
				sync
				cat $WORKDIR/failure.bmp > /dev/fb0
				exit 1
			fi
			sync
		fi

		sync

		if [ -f $WORKDIR/update/camera.tar ]; then
			CAMERA=$WORKDIR/update/camera.tar
			echo "update camera" >> $LOG
			echo "tar camera.tar" >> $LOG
			tar -xvf $WORKDIR/update/camera.tar -C /PROGRAM/library/temp
			echo "tar camera.tar ret=$?" >> $LOG
			if [ $? == 0 ] || [ $? == 141 ]; then
				echo "update camera success" >> $LOG
			else
				echo "update camera failure" >> $LOG
				sync
				cat $WORKDIR/failure.bmp > /dev/fb0
				exit 1
			fi
			sync
		fi

		sync

		if [ -f $WORKDIR/update/openssl-1.0.2d-none.tar ]; then
			OPENSSLLIB=$WORKDIR/update/openssl-1.0.2d-none.tar
			echo "update openssllib" >> $LOG
			echo "tar openssl-1.0.2d-none.tar" >> $LOG
			tar -xvf $WORKDIR/update/openssl-1.0.2d-none.tar -C /PROGRAM/library/temp
			echo "tar openssl-1.0.2d-none.tar ret=$?" >> $LOG
			if [ $? == 0 ] || [ $? == 141 ]; then
				echo "update openssllib success" >> $LOG
			else
				echo "update openssllib failure" >> $LOG
				sync
				cat $WORKDIR/failure.bmp > /dev/fb0
				exit 1
			fi
			sync
		fi

		mv /PROGRAM/library/temp /PROGRAM/library/$VERSION_LIBRARY
	fi
fi

sync

HEX=""
HEX_NAME=""
if [ -f $WORKDIR/$M3HEX -a -f $WORKDIR/IAPAD1IAP ]; then
	HEX=$WORKDIR/$M3HEX
	HEX_NAME=$M3HEX
fi

echo "HEX="$HEX >> $LOG
echo "HEX_NAME="$HEX_NAME >> $LOG

if [ -f $HEX ] && [ $HEX_NAME ]; then
	VER_CONTROL=`ls /PROGRAM/control | head -n 1`
	if [ ! $VER_CONTROL ] || [ ! $VER_CONTROL == $VERSION_CONTROL ] || [ $WORKDIR == "/mnt" ]; then
		echo "rm -rf /PROGRAM/control/*" >> $LOG
		rm -rf /PROGRAM/control/*
		
		echo "update HEX "$HEX_NAME >> $LOG
		echo "create /PROGRAM/control/"$VERSION_CONTROL >> $LOG
		mkdir -p /PROGRAM/control/$VERSION_CONTROL

		cp -f $HEX /PROGRAM/control/$VERSION_CONTROL
		cp -f $WORKDIR/$STM_FIRMWARE_M3 /PROGRAM/control/$VERSION_CONTROL
		sync

		chmod a+x $WORKDIR/IAPAD1IAP
		cp -f $WORKDIR/IAPAD1IAP /PROGRAM/control/$VERSION_CONTROL
		sync

		echo "burn M3 firmware..." >> $LOG
		echo 206 > /sys/class/gpio/export
    		echo "out" > /sys/class/gpio/gpio206/direction
    		echo 1 > /sys/class/gpio/gpio206/value

		/PROGRAM/control/$VERSION_CONTROL/IAPAD1IAP /PROGRAM/control/$VERSION_CONTROL/$HEX_NAME $PORTNAME 57600 /PROGRAM/control/$VERSION_CONTROL/$STM_FIRMWARE_M3
		sync

		if [ $? == 0 ]; then
			echo "update $HEX_NAME success" >> $LOG
			sync
		else
			echo "update $HEX_NAME failure" >> $LOG
			sync
			cat $WORKDIR/failure.bmp > /dev/fb0
			exit 1
		fi

		cd /PROGRAM/control
		ls | grep -v $VERSION_CONTROL | xargs rm -rf
	fi
fi

sync

cp -f $WORKDIR/ffstartup* /PROGRAM
sync

chmod a+x /PROGRAM/ffstartup-arm


if [ -f $WORKDIR/ffcp ]; then
	FFCP=$WORKDIR/ffcp

	echo "update ffcp" >> $LOG
	rm -f /bin/ffcp
	cp -vf $FFCP /bin
	sync
	chmod a+x /bin/ffcp
	echo "update ffcp success" >> $LOG
fi

sync

if [ -f $WORKDIR/AD1-leveling-check.gx ] && [ -f $WORKDIR/AD1-leveling-nut.gx ]; then
	cp -f $WORKDIR/AD1-leveling-check.gx /data
	sync
	
	cp -f $WORKDIR/AD1-leveling-nut.gx /data
	sync

	sed -i '/AD1-leveling-check.gx/d' /opt/flashforge/gcode-sort
	sed -i '/AD1-leveling-nut.gx/d' /opt/flashforge/gcode-sort

	SORT_LINE=`sed -n '$=' /opt/flashforge/gcode-sort`
	echo "H SORT_LINE="$SORT_LINE >> $LOG
	if [ $SORT_LINE -gt 8 ]; then
		SORT_LINE=$((SORT_LINE-8))
		echo "R SORT_LINE="$SORT_LINE >> $LOG
		sed -i '1,'$SORT_LINE'd' /opt/flashforge/gcode-sort
	fi	

	sed -i '01i AD1-leveling-check.gx' /opt/flashforge/gcode-sort
	sed -i '01i AD1-leveling-nut.gx' /opt/flashforge/gcode-sort
	
	sed -i '/AD1-leveling-check.gx/d' /opt/flashforge/has-printed
	sed -i '/AD1-leveling-nut.gx/d' /opt/flashforge/has-printed
fi	

sleep 3


echo "dispaly on lcd" >> $LOG
if [ -f $WORKDIR/complete.bmp ]; then
    cat $WORKDIR/complete.bmp > /dev/fb0
fi

rm -rf /data/update/flashforge_init.sh

echo "export tslib" >> $LOG

#if [ ${#VERSION_LIBRARY} == 0 ]; then
	VERSION_LIBRARY=`ls /PROGRAM/library | head -n 1`
#fi

echo "VERSION_LIBRARY="$VERSION_LIBRARY >> $LOG
sync

export QT_QWS_FONTDIR=/PROGRAM/library/$VERSION_LIBRARY/fonts
export LD_LIBRARY_PATH=/opt/tslib-1.4-none/lib:$LD_LIBRARY_PATH
if [ -f $WORKDIR/play ]; then
	echo "play complete music" >> $LOG
	sync
	chmod a+x $WORKDIR/play
	$WORKDIR/play -qws
fi

rm -rf /data/update/*

halt -f
